/************************************************************************

 Module:      JTVRemote.cpp

 Project:     JTV

 Description: FlyVideo 98 (Temic PAL DK) remote control driver.
 Valid for:

 Prototype (c) 2000 by Ivan Kolin  <ivkolin@mail.ru>
 Copyright (c) 2000 by by Andrew Samusev, <sandys@rbc.ru>

 All rights reserved.

 You may not use any piece of the following code for commercial purposes
 without written permission from the author.

 You may freely copy and distribute the source code as long as it contains
 this copyright notice and the disclaimer. You may freely modify it.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.    IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
#include <unknwn.h>
#include <stdio.h>

#include "jtvrem.h"
#include "jtvbt8x8.h"
#include "btregs.h"
#include "jtvremsk.h"

// This driver corresponds to the JTVRemote Specification 1.2.
// Look at JTVRemSpec1x.zip file for additional information.

// BTTool.exe display for "Animation FlyVideo 98/II":
// GPIO_Dma_Ctl - 0000c004; (0000c007 when TV window open)
// GPIO_Out_En  - 00001800;
// GPIO_Data    - remarked column (for pressed - released
//                keys and loaded/unload4ed JTV).
// When open JTV-window, GPIO_Data values can change it's
// values in regular way that can be used while driver creation.
// Example: 00xxxxxx - 00xxxxxc, where "x" - random value.
// GPIO_Data    - 008de700 (when JTV-window is open)
// GPIO_Data    - 008dff00 (when JTV-window is closed)

const GPIO_Dma_Ctl = 0xc007;
const GPIO_Out_En  = 0x00001800;
const GPIO_Data    = 0x8de700;

KeyDef keys[]=
{
//                                         JTV is loaded         JTV is NOT loaded
// 1-st row keys                           Pressed    Released   Pressed  Released
        { 0x00, "Key_Power", "Power", }, //008de600 - 008de700 / 8dfe00 - 8dff00
// No key
        { 0x80, "Key_MouseMode", "MouseMode", },  //008de680 - 008de700 / 8dfe80 - 8dff00
        { 0x40, "Key_TVFullScreen", "TVFullScreen",},//008de640 - 008de700 / 8dfe40 - 8dff00
// 2-nd row keys
        { 0x48, "Key_ChannelUp","ChannelUp",},//008de648 - 008de700     / 8dfe48 - 008dff00
        { 0xc0, "Key_1", "1", },              //008de6c0 - 008de700 / 008dfec0 - 008dff00
        { 0x20, "Key_2", "2", },              //008de620 - 008de700 / 008dfe20 - 008dff00
        { 0xa0, "Key_3", "3", },              //008de6a0 - 008de700 / 008dfea0 - 008dff00
// ...
        { 0xc8, "Key_ChannelDown","ChannelDown",},//008de6c8 - 008de700 / 008dfec8 - 008dff00
        { 0xe0, "Key_4", "4", },                  //008de6e0 - 008de700 / 008dfee0 - 008dff00
        { 0x10, "Key_5", "5", },                  //008de610 - 008de700 / 008dfe10 - 008dff00
        { 0x90, "Key_6", "6", },                  //008de690 - 008de700 / 008dfe90 - 008dff00

        { 0x28, "Key_VolumeUp","VolumeUp",},  //008de628 - 008de700 / 008dfe28 - 008dff00
        { 0xd0, "Key_7", "7", },              //008de6d0 - 008de700 / 008dfed0 - 008dff00
        { 0x30, "Key_8", "8", },              //008de630 - 008de700 / 008dfe.. - 008dff00
        { 0xb0, "Key_9", "9", },              //008de6b0 - 008de700 / 008dfe.. - 008dff00

        { 0xe8, "Key_VolumeDown","VolumeDown", },//008de6e8 - 008de700 / 008dfe.. - 008dff00
        { 0xf8, "Key_Cancel", "Cancel", },             //008de6f8 - 008de700 / 008dfe.. - 008dff00
        { 0xf0, "Key_0", "0", },                 //008de6f0 - 008de700 / 008dfe.. - 008dff00
        { 0x08, "Key_Enter", "Enter", },         //008de608 - 008de700 / 008dfe.. - 008dff00

        { 0x60, "Key_Recall","Recall",},//008de660 - 008de700 / 008dfe.. - 008dff00
        { 0x70, "Key_AirCableTV", "AirCableTV", },//008de670 - 008de700 / 008dfe.. - 008dff00
        { 0x88, "Key_Video", "Video", },     //008de688 - 008de700 / 008dfe.. - 008dff00
        { 0x50, "Key_Inc", "Inc", },       //008de650 - 008de700 / 008dfe.. - 008dff00

        { 0x48, "Key_Left", "Left", },  //008de648 - 008de700 / 008dfe.. - 008dff00
        { 0xc8, "Key_Empty", "Empty", },     //008de6c8 - 008de700 / 008dfe.. - 008dff00
        { 0xa8, "Key_Audio", "Right", },//008de6a8 - 008de700 / 008dfe.. - 008dff00
        { 0x68, "Key_Dec", "Dec", },       //008de668 - 008de700 / 008dfe.. - 008dff00

        { 0xb8, "Key_Window", "Window", },   //008de6b8 - 008de700 / 008dfe.. - 008dff00
        { 0x98, "Key_Down", "Down", },     //008de698 - 008de700 / 008dfe.. - 008dff00
        { 0x58, "Key_Esc", "Esc", },      //008de658 - 008de700 / 008dfe.. - 008dff00
// No key

        { 0x38, "Key_TabRight", "TabRight", }, //008de638 - 008de700 / 008dfe.. - 008dff00
        { 0x78, "Key_TabLeft", "TabLeft", },  //008de678 - 008de700 / 008dfe.. - 008dff00
        { 0xd8, "Key_Mute", "Mute", },     //008de6d8 - 008de700 / 008dfe.. - 008dff00
        { 0x18, "Key_AVSource", "AVSource", }, //008de618 - 008de700 / 008dfe.. - 008dff00
};


typedef UINT (CALLBACK* LPFNDLLFUNC1)(DWORD,UINT);


DWORD dwParam1;
UINT  uParam2, uReturnVal;

INT first_delay=200;    // default first delay for a key
INT repeat_delay=20;    // default repeat delay for a key


void log(const char* str, ...)
{
  va_list arglist;

  FILE* l=fopen("JTVRemote.log","a+t");
  if (l==NULL) return;
  va_start( arglist, str );
  vfprintf(l, str, arglist );
  va_end( arglist );
  fclose(l);

// Look at section [JTVRemote] in "jtv.ini" file, look for strings
// "KeyFirstDelay=" and "KeyRepeatDelay=", example:
//      [JTVRemote]
//      ............
//      KeyFirstDelay=800
//      KeyRepeatDelay=60
// If strings exist, get corresponded values into "first_delay"
// and "repeat_delay" variables, if not - keep default values above.

  first_delay=GetPrivateProfileInt("JTVRemote","KeyFirstDelay",first_delay,".\\jtv.ini");
  repeat_delay=GetPrivateProfileInt("JTVRemote","KeyRepeatDelay",repeat_delay,".\\jtv.ini");

}

class JTVRemote : public CRemoteDriver
{

public:

        JTVRemote(KeyDef *k, int count) : CRemoteDriver(k, count)
                {
                log("> Initializing FlyVideo 98/II remote (with MouseMode button) module.\n");
                log("> Copyright (c) 2000 by Andrew Samusev, <sandys@rbc.ru>.\n");
                log("> KeyFirstDelay=%d, KeyRepeatDelay=%d\n",first_delay,repeat_delay);
                };



        void PollingThread()
        {
                IBT8x8Device *bt;
                DWORD last_code=0xffffffff;;
                DWORD last_tick=GetTickCount();

                if (SUCCEEDED(device->QueryInterface(IID_IBT8x8Device, (void **)&bt)))
                {
                    DWORD value;
                    INT key_pause=20; // initial repeat delay
                    DWORD test_code=0;

                    bt->ReadBT(BT848_GPIO_DMA_CTL, &value);
                    bt->WriteBT(BT848_GPIO_DMA_CTL, value | GPIO_Dma_Ctl);

                    bt->ReadBT(BT848_GPIO_OUT_EN, &value);
                    bt->WriteBT(BT848_GPIO_OUT_EN, (value | GPIO_Out_En));

                    bt->ReadBT(BT848_GPIO_DATA, &value);
                    bt->WriteBT(BT848_GPIO_DATA, value | GPIO_Data);


                    while (WaitForSingleObject(hBreakEvent, 0) != WAIT_OBJECT_0)
                    {
                         Sleep(key_pause);

                         DWORD gpio_data=0;
                         DWORD value=0;
                         DWORD key_flag=0;

                         bt->ReadBT(BT848_GPIO_DATA, &gpio_data);

                         value = (gpio_data &0x000000ff);
                         key_flag = ((gpio_data &0x0000ff00)>>8);

                                 // Test block begin - comment all while build checked out driver
                                 //if (test_code != value)
                                 //{
                                 //       log(">> ---- Test information ---- \n");
                                 //       log(">>      Tick_Counter: %d\n",GetTickCount());
                                 //       log(">>         GPIO_DATA: %x\n",gpio_data);
                                 //       log(">>  Current_Key_Code: %x\n",value);
                                 //       log(">> Previous_Key_Code: %x\n",last_code);
                                 //
                                 //       log(">>    Press_Key_Flag: %x\n",key_flag);
                                 //       test_code = value;
                                 //}
                                 // Test block end

                         if ((key_flag == 0xe6) | (key_flag == 0xfe)) // any key is pressed
                         {
                             if (last_code == value)
                             {     // auto repeat for any pressed key
                                   //     log(">>    Press_Key_Auto: %x\n",value); //for test
                                   AddStroke(last_code);
                                   key_pause = repeat_delay;
                             }
                             else
                             {     // first run for any pressed key
                                   //     log(">>   Press_Key_First: %x\n",value); //for test
                                   last_code = value;
                                   AddStroke(last_code);
                                   key_pause = first_delay;
                             }
                         }
                         else  // no one key is pressed
                         {
                             if ((value!=last_code) && (last_code!=0xffffffff))
                             {
                                last_code = value;
                             }

                             key_pause = 20;
                         };

                    };

                log("> Break event received. Bye!\n");

                bt->Release();
                };
        };
};

extern "C"
{
        __declspec(dllexport) IJTVRemoteControl *__stdcall GetInterfacePtr()
        {
                return new JTVRemote(keys, sizeof(keys) / sizeof(keys[0]));
        };
};
